/*
 Copyright (c) 2009 [Joerg Ruedenauer]
 
 This file is part of RPGSoundController.

 RPGSoundController is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 RPGSoundController is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with RPGSoundController; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package rpgsoundcontroller.data;

import java.io.File;
import java.io.IOException;

import javax.swing.KeyStroke;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import rpgsoundcontroller.messages.Messages;
import rpgsoundcontroller.messages.Message.MessageType;
import rpgsoundcontroller.util.Localization;

public final class FileParser {

  public static Configuration parseFile(File file) {
    Configuration config = new Configuration();
    
    try {
      DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
      DocumentBuilder db = dbf.newDocumentBuilder();
      Document dom = db.parse(file);

      Element docEle = dom.getDocumentElement();
      
      NodeList nl = docEle.getElementsByTagName("Mode"); //$NON-NLS-1$
      for(int i = 0 ; i < nl.getLength();i++) {
        Element el = (Element)nl.item(i);
        Mode mode = readMode(el);
        if (mode == null) continue;
        if (config.containsKeyStroke(mode.getKeyStroke())) {
          Messages.addMessage(MessageType.Error, Localization.getString("FileParser.DuplicateKey") + mode.getKeyStroke() + Localization.getString("FileParser.InConfiguration")); //$NON-NLS-1$ //$NON-NLS-2$
          Messages.addMessage(MessageType.Error, Localization.getString("FileParser.Mode") + mode.getTitle() + Localization.getString("FileParser.Ignored")); //$NON-NLS-1$ //$NON-NLS-2$
          continue;
        }
        config.addMode(mode);
      }
      return config;
    }
    catch(ParserConfigurationException pce) {
      Messages.addMessage(MessageType.Error, pce.getLocalizedMessage());
      return null;
    }
    catch(SAXException se) {
      Messages.addMessage(MessageType.Error, se.getLocalizedMessage());
      return null;
    }
    catch (IOException e) {
      Messages.addMessage(MessageType.Error, e.getLocalizedMessage());
      return null;
    }
  }
  
  private static Mode readMode(Element el) {
    String title = el.getAttribute("title"); //$NON-NLS-1$
    if (title == null) {
      Messages.addMessage(MessageType.Error, Localization.getString("FileParser.TitleMissing")); //$NON-NLS-1$
      return null;
    }
    KeyStroke keyStroke = KeyStroke.getKeyStroke(el.getAttribute("key")); //$NON-NLS-1$
    if (keyStroke == null) {
      Messages.addMessage(MessageType.Error, Localization.getString("FileParser.KeyOfMode") + title + Localization.getString("FileParser.MissingModeIgnored")); //$NON-NLS-1$ //$NON-NLS-2$
      return null;
    }
    
    boolean isNumber = false;
    for (int i = 0; i < 9; ++i) {
      if (keyStroke.equals(KeyStroke.getKeyStroke("" + i))) { //$NON-NLS-1$
        isNumber = true;
        break;
      }
    }
    if (!isNumber) {
      Messages.addMessage(MessageType.Warning, Localization.getString("FileParser.Mode") + title + Localization.getString("FileParser.noNumberKey")); //$NON-NLS-1$ //$NON-NLS-2$
    }
    
    Mode mode = new Mode(title, keyStroke);
    
    NodeList nl = el.getElementsByTagName("Command"); //$NON-NLS-1$
    
    if (nl.getLength() == 0) {
      Messages.addMessage(MessageType.Warning, Localization.getString("FileParser.Mode") + title + Localization.getString("FileParser.NoCommands")); //$NON-NLS-1$ //$NON-NLS-2$
    }
    
    for(int i = 0 ; i < nl.getLength();i++) {
      Element ele = (Element)nl.item(i);
      Command command = readCommand(ele);
      if (command == null) continue;
      if (mode.containsKeyStroke(command.getKeyStroke())) {
        Messages.addMessage(MessageType.Error, Localization.getString("FileParser.DuplicateKey") + command.getKeyStroke() + Localization.getString("FileParser.InMode") + title + "."); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
        Messages.addMessage(MessageType.Error, Localization.getString("FileParser.Command") + command.getTitle() + Localization.getString("FileParser.Ignored")); //$NON-NLS-1$ //$NON-NLS-2$
        continue;
      }
      mode.addCommand(command);
    }
    
    return mode;
  }
  
  private static Command readCommand(Element el) {
    String title = el.getAttribute("title"); //$NON-NLS-1$
    if (title == null) {
      Messages.addMessage(MessageType.Error, Localization.getString("FileParser.CommandTitleMissing")); //$NON-NLS-1$
      return null;
    }
    KeyStroke keyStroke = KeyStroke.getKeyStroke(el.getAttribute("key")); //$NON-NLS-1$
    if (keyStroke == null) {
      Messages.addMessage(MessageType.Error, Localization.getString("FileParser.KeyStrokeCommand") + title + Localization.getString("FileParser.MissingCommandIgnored")); //$NON-NLS-1$ //$NON-NLS-2$
      return null;
    }
    String continuous = el.getAttribute("continuous"); //$NON-NLS-1$
    boolean isContinuous = "true".equals(continuous); //$NON-NLS-1$
    String group = el.getAttribute("group"); //$NON-NLS-1$
    int theGroup = 0;
    try {
      theGroup = group != null && group.length() > 0 ? Integer.parseInt(group) : 0;
    }
    catch (NumberFormatException e) {
      Messages.addMessage(MessageType.Error, Localization.getString("FileParser.GroupCommand") + title + Localization.getString("FileParser.NoNumber")); //$NON-NLS-1$ //$NON-NLS-2$
    }
    if (theGroup == -1) {
      Messages.addMessage(MessageType.Warning, Localization.getString("FileParser.GroupCommand") + title + Localization.getString("FileParser.ReservedforModes")); //$NON-NLS-1$ //$NON-NLS-2$
    }
    String stopsAll = el.getAttribute("stopsAll"); //$NON-NLS-1$
    boolean doesStopAll = "true".equals(stopsAll); //$NON-NLS-1$
    return new Command(title, keyStroke, isContinuous, theGroup, doesStopAll);
  }
  
}
