/*
 Copyright (c) 2009 [Joerg Ruedenauer]
 
 This file is part of RPGSoundController.

 RPGSoundController is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 RPGSoundController is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with RPGSoundController; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package rpgsoundcontroller.gui;

import java.awt.Dimension;
import javax.swing.JPanel;
import javax.swing.BorderFactory;
import javax.swing.border.EtchedBorder;
import javax.swing.border.TitledBorder;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.JButton;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.prefs.Preferences;

import javax.swing.JComboBox;
import javax.swing.JCheckBox;

import rpgsoundcontroller.control.Control;
import rpgsoundcontroller.control.Version;
import rpgsoundcontroller.data.Command;
import rpgsoundcontroller.data.Configuration;
import rpgsoundcontroller.data.Mode;
import rpgsoundcontroller.gui.util.ExampleFileFilter;
import rpgsoundcontroller.messages.IMessageListener;
import rpgsoundcontroller.messages.Message;
import rpgsoundcontroller.messages.Messages;
import rpgsoundcontroller.messages.Message.MessageType;
import rpgsoundcontroller.network.IServerListener;
import rpgsoundcontroller.network.ServerInfo;
import rpgsoundcontroller.network.ServerSearch;
import rpgsoundcontroller.util.Directories;
import rpgsoundcontroller.util.Localization;

public final class MainFrame extends FrameController implements IMessageListener, IServerListener {

  private JPanel jContentPane = null;
  private JPanel jPanel = null;
  private JLabel jLabel = null;
  private JTextField fileField = null;
  private JButton fileButton = null;
  private JPanel jPanel1 = null;
  private JLabel jLabel1 = null;
  private JComboBox serverBox = null;
  private JButton connectButton = null;
  private JPanel jPanel2 = null;
  private JCheckBox messagesBox = null;
  private JCheckBox controlsBox = null;
  private JButton settingsButton = null;
  private JCheckBox modesBox = null;
  private JButton quitButton = null;
  
  private ServerSearch serverSearch = null;
  
  private final class MyWindowListener extends WindowAdapter {

    public void windowIconified(WindowEvent e) {
      FrameManagement.getInstance().iconifyAllFrames(MainFrame.this);
    }

    public void windowDeiconified(WindowEvent e) {
      FrameManagement.getInstance().deiconifyAllFrames(MainFrame.this);
    }

  }

  /**
   * This method initializes 
   * 
   */
  public MainFrame() {
  	super(Localization.getString("MainFrame.SoundController")); //$NON-NLS-1$
  	initialize();
    this.addWindowListener(new MyWindowListener());
  	Messages.getInstance().addObserver(this);
  	Messages.addMessage(MessageType.Info, Localization.getString("MainFrame.RSCVersion") + Version.getCurrentVersionString() + Localization.getString("MainFrame.started")); //$NON-NLS-1$ //$NON-NLS-2$
  	ModeFrames.getInstance();
  	addButton(Localization.getString("MainFrame.Messages"), getMessagesBox()); //$NON-NLS-1$
  	addButton(Localization.getString("MainFrame.MainControls"), getControlsBox()); //$NON-NLS-1$
  	addButton(Localization.getString("MainFrame.Modes"), getModesBox()); //$NON-NLS-1$
  	serverSearch = new ServerSearch(this, Preferences.userNodeForPackage(MainFrame.class).getInt("UDPPort", 8009)); //$NON-NLS-1$
  	serverSearch.startSearch();
  }
  
  public void dispose() {
    storeLayout();
    Control.getInstance().disconnect();
    Messages.getInstance().removeObserver(this);
    FrameManagement.getInstance().setExiting();
    FrameManagement.getInstance().closeAllFrames(this);
    super.dispose();
  }

  /**
   * This method initializes this
   * 
   */
  private void initialize() {
        this.setSize(new Dimension(456, 215));
        this.setContentPane(getJContentPane());
        this.setTitle(Localization.getString("MainFrame.SoundController")); //$NON-NLS-1$
  }
  
  private void storeLayout() {
    if (Control.getInstance().getConfiguration() != null) {
      File layoutsFile = new File(Directories.getUserDataPath() + Control.getInstance().getFileName() + ".layout"); //$NON-NLS-1$
      try {
        FrameLayouts.getInstance().storeToFile(layoutsFile.getAbsolutePath());
      }
      catch (IOException e) {
        Messages.addMessage(MessageType.Warning, Localization.getString("MainFrame.LayoutWriteError")); //$NON-NLS-1$
      }
    }    
  }
  
  public void openFile(String fileName) {
    File file = new File(fileName);
    if (!file.exists()) return;
    
    storeLayout();

    List<SubFrame> openFrames = new ArrayList<SubFrame>();
    openFrames.add(this);
    if (messageFrame != null) openFrames.add(messageFrame);
    if (controlsFrame != null) openFrames.add(controlsFrame);
    SubFrame[] frames = new SubFrame[openFrames.size()];
    openFrames.toArray(frames);
    FrameManagement.getInstance().closeAllFrames(frames);
      
    Control.getInstance().openFile(file);
    if (Control.getInstance().getConfiguration() != null) {
      getModesBox().setEnabled(true);
      getFileField().setText(file.getName());
      
      File layoutsFile = new File(Directories.getUserDataPath() + file.getName() + ".layout"); //$NON-NLS-1$
      if (layoutsFile.exists()) try {
        FrameLayouts.getInstance().readFromFile(layoutsFile.getAbsolutePath());
        FrameLayouts.getInstance().restoreLastLayout();
      }
      catch (IOException e) {
        Messages.addMessage(MessageType.Warning, Localization.getString("MainFrame.LayoutReadError")); //$NON-NLS-1$
      }
      
      Preferences.userNodeForPackage(MainFrame.class).put("LastConfiguration", fileName); //$NON-NLS-1$
    }
    else {
      getModesBox().setEnabled(false);
      getFileField().setText(""); //$NON-NLS-1$
    }
  }

  /**
   * This method initializes jContentPane	
   * 	
   * @return javax.swing.JPanel	
   */
  private JPanel getJContentPane() {
    if (jContentPane == null) {
      jContentPane = new JPanel();
      jContentPane.setLayout(new BoxLayout(jContentPane, BoxLayout.LINE_AXIS));
      jContentPane.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
      JPanel leftPanel = new JPanel();
      leftPanel.setLayout(new BoxLayout(leftPanel, BoxLayout.PAGE_AXIS));
      leftPanel.add(getJPanel());
      leftPanel.add(Box.createRigidArea(new Dimension(0, 5)));
      leftPanel.add(getJPanel1());
      jContentPane.add(leftPanel);
      jContentPane.add(Box.createRigidArea(new Dimension(5, 0)));
      JPanel rightPanel = new JPanel();
      rightPanel.setLayout(new BoxLayout(rightPanel, BoxLayout.PAGE_AXIS));
      rightPanel.add(getJPanel2());
      rightPanel.add(Box.createRigidArea(new Dimension(0, 5)));
      JPanel buttonPanel = new JPanel();
      buttonPanel.setLayout(new GridLayout(2, 1, 5, 5));
      buttonPanel.add(getSettingsButton());
      buttonPanel.add(getQuitButton());
      JPanel rightLowerPanel = new JPanel();
      rightLowerPanel.setLayout(new BorderLayout());
      rightLowerPanel.add(buttonPanel, BorderLayout.NORTH);
      rightPanel.add(rightLowerPanel);
      jContentPane.add(rightPanel);
    }
    return jContentPane;
  }

  /**
   * This method initializes jPanel	
   * 	
   * @return javax.swing.JPanel	
   */
  private JPanel getJPanel() {
    if (jPanel == null) {
      jLabel = new JLabel();
      jLabel.setText(Localization.getString("MainFrame.File")); //$NON-NLS-1$
      jPanel = new JPanel();
      jPanel.setLayout(new BorderLayout());
      JPanel leftPanel = new JPanel();
      leftPanel.setLayout(new BoxLayout(leftPanel, BoxLayout.PAGE_AXIS));
      leftPanel.add(Box.createRigidArea(new Dimension(0, 5)));
      leftPanel.add(jLabel);
      leftPanel.add(Box.createVerticalGlue());
      jPanel.add(leftPanel, BorderLayout.WEST);
      JPanel rightPanel = new JPanel();
      rightPanel.setLayout(new BoxLayout(rightPanel, BoxLayout.PAGE_AXIS));
      rightPanel.setBorder(BorderFactory.createEmptyBorder(0, 5, 5, 5));
      rightPanel.add(Box.createRigidArea(new Dimension(0, 5)));
      JPanel fieldPanel = new JPanel();
      fieldPanel.setLayout(new BoxLayout(fieldPanel, BoxLayout.LINE_AXIS));
      fieldPanel.add(Box.createRigidArea(new Dimension(5, 0)));
      fieldPanel.add(getFileField());
      rightPanel.add(fieldPanel);
      rightPanel.add(Box.createVerticalStrut(5));
      JPanel buttonPanel = new JPanel(new FlowLayout(FlowLayout.LEADING));
      buttonPanel.add(getFileButton());
      rightPanel.add(buttonPanel);
      JPanel centerPanel = new JPanel(new BorderLayout());
      centerPanel.add(rightPanel, BorderLayout.NORTH);
      jPanel.add(centerPanel, BorderLayout.CENTER);
      jPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED), Localization.getString("MainFrame.Configuration"), TitledBorder.DEFAULT_JUSTIFICATION, TitledBorder.DEFAULT_POSITION, null, null)); //$NON-NLS-1$
    }
    return jPanel;
  }

  /**
   * This method initializes fileField	
   * 	
   * @return javax.swing.JTextField	
   */
  private JTextField getFileField() {
    if (fileField == null) {
      fileField = new JTextField();
      fileField.setEditable(false);
    }
    return fileField;
  }

  /**
   * This method initializes fileButton	
   * 	
   * @return javax.swing.JButton	
   */
  private JButton getFileButton() {
    if (fileButton == null) {
      fileButton = new JButton();
      fileButton.setText(Localization.getString("MainFrame.Select")); //$NON-NLS-1$
      fileButton.addActionListener(new java.awt.event.ActionListener() {
        public void actionPerformed(java.awt.event.ActionEvent e) {
          selectFile();
        }
      });
    }
    return fileButton;
  }
  
  private void selectFile() {
    JFileChooser chooser = new JFileChooser();
    chooser.setAcceptAllFileFilterUsed(true);
    chooser.setFileFilter(new ExampleFileFilter("xml", Localization.getString("MainFrame.ConfigFiles"))); //$NON-NLS-1$ //$NON-NLS-2$
    chooser.setCurrentDirectory(Directories.getLastUsedDirectory(this, "ConfigurationFiles")); //$NON-NLS-1$
    chooser.setMultiSelectionEnabled(false);
    if (chooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
      File file = chooser.getSelectedFile();
      Directories.setLastUsedDirectory(this, "ConfigurationFiles", file); //$NON-NLS-1$
      openFile(file.getAbsolutePath());
    }
  }

  /**
   * This method initializes jPanel1	
   * 	
   * @return javax.swing.JPanel	
   */
  private JPanel getJPanel1() {
    if (jPanel1 == null) {
      jLabel1 = new JLabel();
      jLabel1.setText(Localization.getString("MainFrame.Server")); //$NON-NLS-1$
      jPanel1 = new JPanel();
      jPanel1.setLayout(new BorderLayout());
      JPanel leftPanel = new JPanel();
      leftPanel.setLayout(new BoxLayout(leftPanel, BoxLayout.PAGE_AXIS));
      leftPanel.add(Box.createRigidArea(new Dimension(0, 5)));
      leftPanel.add(jLabel1);
      leftPanel.add(Box.createVerticalGlue());
      jPanel1.add(leftPanel, BorderLayout.WEST);
      JPanel rightPanel = new JPanel();
      rightPanel.setLayout(new BoxLayout(rightPanel, BoxLayout.PAGE_AXIS));
      rightPanel.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
      JPanel fieldPanel = new JPanel();
      fieldPanel.setLayout(new BoxLayout(fieldPanel, BoxLayout.LINE_AXIS));
      fieldPanel.add(Box.createRigidArea(new Dimension(5, 0)));
      fieldPanel.add(getServerBox());
      rightPanel.add(fieldPanel);
      rightPanel.add(Box.createRigidArea(new Dimension(0, 5)));
      JPanel buttonPanel = new JPanel(new FlowLayout(FlowLayout.LEADING));
      buttonPanel.add(getConnectButton());
      rightPanel.add(buttonPanel);
      JPanel centerPanel = new JPanel(new BorderLayout());
      centerPanel.add(rightPanel, BorderLayout.NORTH);
      jPanel1.add(centerPanel, BorderLayout.CENTER);
      jPanel1.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED), Localization.getString("MainFrame.Network"), TitledBorder.DEFAULT_JUSTIFICATION, TitledBorder.DEFAULT_POSITION, null, null)); //$NON-NLS-1$ //$NON-NLS-2$
    }
    return jPanel1;
  }

  /**
   * This method initializes serverBox	
   * 	
   * @return javax.swing.JComboBox	
   */
  private JComboBox getServerBox() {
    if (serverBox == null) {
      serverBox = new JComboBox();
    }
    return serverBox;
  }

  /**
   * This method initializes connectButton	
   * 	
   * @return javax.swing.JButton	
   */
  private JButton getConnectButton() {
    if (connectButton == null) {
      connectButton = new JButton();
      connectButton.setEnabled(false);
      connectButton.setText(Localization.getString("MainFrame.Connect")); //$NON-NLS-1$
      connectButton.addActionListener(new java.awt.event.ActionListener() {
        public void actionPerformed(java.awt.event.ActionEvent e) {
          connectOrDisconnect();
        }
      });
    }
    return connectButton;
  }
  
  private void connectOrDisconnect() {
    if (Control.getInstance().isConnected()) {
      Control.getInstance().disconnect();
    }
    else {
      String server = getServerBox().getSelectedItem().toString();
      ServerInfo serverInfo = servers.get(server);
      if (serverInfo != null) {
        Control.getInstance().connect(serverInfo);
      }
    }
    if (Control.getInstance().isConnected()) {
      serverSearch.stopSearch();
      getConnectButton().setText(Localization.getString("MainFrame.Disconnect")); //$NON-NLS-1$
    }
    else {
      servers.clear();
      getServerBox().removeAllItems();
      getConnectButton().setEnabled(false);
      getConnectButton().setText(Localization.getString("MainFrame.Connect")); //$NON-NLS-1$
      serverSearch.startSearch();
    }
  }

  /**
   * This method initializes jPanel2	
   * 	
   * @return javax.swing.JPanel	
   */
  private JPanel getJPanel2() {
    if (jPanel2 == null) {
      jPanel2 = new JPanel();
      jPanel2.setLayout(new BorderLayout());
      jPanel2.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED), Localization.getString("MainFrame.Windows"), TitledBorder.DEFAULT_JUSTIFICATION, TitledBorder.DEFAULT_POSITION, null, null)); //$NON-NLS-1$
      JPanel boxPanel = new JPanel();
      boxPanel.setLayout(new GridLayout(3, 1, 5, 5));
      boxPanel.add(getMessagesBox());
      boxPanel.add(getControlsBox());
      boxPanel.add(getModesBox());
      jPanel2.add(boxPanel, BorderLayout.CENTER);
    }
    return jPanel2;
  }

  /**
   * This method initializes messagesBox	
   * 	
   * @return javax.swing.JCheckBox	
   */
  private JCheckBox getMessagesBox() {
    if (messagesBox == null) {
      messagesBox = new JCheckBox();
      messagesBox.setText(Localization.getString("MainFrame.Messages")); //$NON-NLS-1$
      messagesBox.addActionListener(new java.awt.event.ActionListener() {
        public void actionPerformed(java.awt.event.ActionEvent e) {
          if (messageFrame != null && messageFrame.isVisible()) {
            messageFrame.dispose();
            messageFrame = null;
          }
          else {
            messageFrame = new MessageFrame();
            messageFrame.addWindowListener(new WindowAdapter() {
              public void windowClosing(java.awt.event.WindowEvent e) {
                MainFrame.this.getMessagesBox().setSelected(false);
              }
            });
            messageFrame.setVisible(true);
          }
        }

      });
    }
    return messagesBox;
  }
  
  private MessageFrame messageFrame = null;

  /**
   * This method initializes controlsBox	
   * 	
   * @return javax.swing.JCheckBox	
   */
  private JCheckBox getControlsBox() {
    if (controlsBox == null) {
      controlsBox = new JCheckBox();
      controlsBox.setText(Localization.getString("MainFrame.MainControls")); //$NON-NLS-1$
      controlsBox.addActionListener(new java.awt.event.ActionListener() {
        public void actionPerformed(java.awt.event.ActionEvent e) {
          if (controlsFrame != null && controlsFrame.isVisible()) {
            controlsFrame.dispose();
            controlsFrame = null;
          }
          else {
            controlsFrame = new MainControlsFrame();
            controlsFrame.addWindowListener(new WindowAdapter() {
              public void windowClosing(java.awt.event.WindowEvent e) {
                MainFrame.this.getControlsBox().setSelected(false);
              }
            });
            controlsFrame.setVisible(true);
          }
        }
      });
    }
    return controlsBox;
  }
  
  private MainControlsFrame controlsFrame = null;

  /**
   * This method initializes jButton	
   * 	
   * @return javax.swing.JButton	
   */
  private JButton getSettingsButton() {
    if (settingsButton == null) {
      settingsButton = new JButton();
      settingsButton.setText(Localization.getString("MainFrame.Settings")); //$NON-NLS-1$
      settingsButton.addActionListener(new java.awt.event.ActionListener() {
        public void actionPerformed(java.awt.event.ActionEvent e) {
          int oldPort = Preferences.userNodeForPackage(MainFrame.class).getInt("UDPPort", 8009);  //$NON-NLS-1$
          OptionsDialog dialog = new OptionsDialog(MainFrame.this);
          dialog.setModal(true);
          dialog.setVisible(true);
          int newPort = Preferences.userNodeForPackage(MainFrame.class).getInt("UDPPort", 8009);  //$NON-NLS-1$
          if (serverSearch != null && oldPort != newPort) {
            boolean wasSearching = serverSearch.isSearching();
            if (wasSearching) {
              serverSearch.stopSearch();
            }
            serverSearch.dispose();
            serverSearch = new ServerSearch(MainFrame.this, newPort);
            if (wasSearching) {
              serverSearch.startSearch();
            }
          }
        }
      });
    }
    return settingsButton;
  }

  /**
   * This method initializes modesBox	
   * 	
   * @return javax.swing.JCheckBox	
   */
  private JCheckBox getModesBox() {
    if (modesBox == null) {
      modesBox = new JCheckBox();
      modesBox.setEnabled(false);
      modesBox.setText(Localization.getString("MainFrame.Modes")); //$NON-NLS-1$
      modesBox.addActionListener(new java.awt.event.ActionListener() {
        public void actionPerformed(java.awt.event.ActionEvent e) {
          if (modesFrame != null && modesFrame.isVisible()) {
            modesFrame.dispose();
            modesFrame = null;
          }
          else {
            Configuration config = Control.getInstance().getConfiguration();
            if (config == null) return;
            List<Mode> modes = config.getModes();
            if (modes.size() == 0) return;
            List<Command> commands = new ArrayList<Command>();
            commands.addAll(modes);
            modesFrame = new ModesFrame(commands);
            modesFrame.addWindowListener(new WindowAdapter() {
              public void windowClosing(java.awt.event.WindowEvent e) {
                MainFrame.this.getMessagesBox().setSelected(false);
              }
            });
            modesFrame.setVisible(true);
          }
        }
      });
    }
    return modesBox;
  }
  
  private ModesFrame modesFrame = null;

  /**
   * This method initializes quitButton	
   * 	
   * @return javax.swing.JButton	
   */
  private JButton getQuitButton() {
    if (quitButton == null) {
      quitButton = new JButton();
      quitButton.setText(Localization.getString("MainFrame.Quit")); //$NON-NLS-1$
      quitButton.addActionListener(new java.awt.event.ActionListener() {
        public void actionPerformed(java.awt.event.ActionEvent e) {
          dispose();
        }
      });
    }
    return quitButton;
  }

  /* (non-Javadoc)
   * @see rpgsoundcontroller.messages.IMessageListener#messageAdded(rpgsoundcontroller.messages.Message)
   */
  @Override
  public void messageAdded(Message message) {
    if (message.getType().ordinal() <= Message.MessageType.Warning.ordinal()) {
      int messageLevel = Preferences.userNodeForPackage(MainFrame.class).getInt("MessageLevel", Message.MessageType.Warning.ordinal()); //$NON-NLS-1$
      if (message.getType().ordinal() <= messageLevel) {
        if (!getMessagesBox().isSelected()) {
          getMessagesBox().doClick();
        }
      }
    }
  }

  /* (non-Javadoc)
   * @see rpgsoundcontroller.network.IServerListener#serverFound(rpgsoundcontroller.network.ServerInfo)
   */
  @Override
  public void serverFound(ServerInfo server) {
    if (servers.containsKey(server.getName())) return;
    Messages.addMessage(MessageType.Info, Localization.getString("MainFrame.ServerFound") + server.getName() + Localization.getString("MainFrame.33")); //$NON-NLS-1$ //$NON-NLS-2$
    getServerBox().addItem(server.getName());
    getConnectButton().setEnabled(true);
    servers.put(server.getName(), server);
  }
  
  private Map<String, ServerInfo> servers = new HashMap<String, ServerInfo>();
  
  public int getHierarchyPosition() {
    return 0;
  }
  

}  //  @jve:decl-index=0:visual-constraint="10,10"
